"""
WebSwitch(Plus) from https://www.controlbyweb.com/
Refer to the manual page 73-80
"""
import pymodbus as mb
from pymodbus.client.sync import ModbusTcpClient as ModbusClient
from time import sleep


class WebSwitch:
    def __init__(self, host, port):
        self.host = host
        self.port = port

    def __enter__(self):
        self.webctrl = mb.client.sync.ModbusTcpClient(self.host, self.port)
        self.webctrl.connect()
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.webctrl.close()

    def send_write_coil(self, address, value):
        return self.webctrl.write_coil(address, value)

    def send_read_coils(self, address, count=4):
        return self.webctrl.read_coils(address, count)  # Fixed count for 2 outlets and 2 inputs

    def switch_on_outlet1(self):
        self.send_write_coil(0, 1)

    def switch_off_outlet1(self):
        self.send_write_coil(0, 0)

    def switch_on_outlet2(self):
        self.send_write_coil(1, 1)

    def switch_off_outlet2(self):
        self.send_write_coil(1, 0)


if __name__ == '__main__':
    with WebSwitch('192.168.1.2', '502') as wc:  # default address at 192.168.1.2
        print(wc.send_read_coils(0).bits)
        # for _ in range(10):
        #     print(wc.send_write_coil(0, 1 if _ % 2 == 0 else 0))
        #     print(wc.send_write_coil(1, 1 if _ % 2 == 0 else 0))
        #     sleep(5)
        wc.switch_on_outlet1()
        sleep(5)
        wc.switch_off_outlet1()
        sleep(5)
        wc.switch_on_outlet2()
        sleep(5)
        wc.switch_off_outlet2()



from qcodes.instrument.ip import IPInstrument
import qcodes.utils.validators as vals


class AmSsm23q(IPInstrument):
    SCL_OPCODE = chr(0) + chr(7)
    DI_LOWER = 0
    DI_UPPER = 95000  # MAX=95197 approx. 95000/20000=4.75 revolution

    def __init__(self, address=None, port=7776, **kwargs):
        super().__init__('AmSsm23q_', address, port, timeout=5, terminator='\r', persistent=False,
                         write_confirmation=True, **kwargs)
        self.add_parameter('init_mode',
                           label='Initial power up mode setting',
                           set_cmd=self.SCL_OPCODE + 'PM{}',
                           get_cmd=self.SCL_OPCODE+'PM',
                           get_parser=lambda s: s.partition('=')[2])

        self.add_parameter('alarm_code',
                           label='Alarm code state',
                           set_cmd=False,
                           get_cmd=self.SCL_OPCODE+'AL',
                           get_parser=lambda s: s.partition('=')[2])

        self.add_parameter('request_status',
                           label='Request status state',
                           set_cmd=False,
                           get_cmd=self.SCL_OPCODE+'RS',
                           get_parser=lambda s: s.partition('=')[2])

        self.add_parameter('status_code',
                           label='Status code',
                           set_cmd=False,
                           get_cmd=self.SCL_OPCODE+'SC',
                           get_parser=lambda s: s.partition('=')[2])

        self.add_parameter('immediate_analog',
                           label='Immediate analog input',
                           set_cmd=False,
                           get_cmd=self.SCL_OPCODE+'IA',
                           get_parser=lambda s: s.partition('=')[2])

        self.add_parameter('alarm_reset',
                           label='Alarm reset',
                           set_cmd=self.SCL_OPCODE+'AR',
                           get_cmd=False,
                           get_parser=lambda s: s.partition('=')[2])

        self.add_parameter('motor_enable',
                           label='Motor enable',
                           set_cmd=self.SCL_OPCODE+'ME',
                           get_cmd=False,
                           get_parser=lambda s: s.partition('=')[2])

        self.add_parameter('distance',
                           label='Distance or position setting',
                           set_cmd=self.SCL_OPCODE+'DI{}',
                           get_cmd=self.SCL_OPCODE+'DI',
                           vals=vals.Numbers(self.DI_LOWER, self.DI_UPPER),
                           get_parser=lambda s: s.partition('=')[2])

        self.add_parameter('feed_length',
                           label='Feed relative distance length',
                           set_cmd=self.SCL_OPCODE+'FL',
                           get_cmd=False,
                           get_parser=lambda s: s.partition('=')[2],
                           post_delay=0)

        self.add_parameter('feed_position',
                           label='Feed absolute position',
                           set_cmd=self.SCL_OPCODE+'FP{}',
                           get_cmd=False,
                           set_parser=int,
                           get_parser=lambda s: s.partition('=')[2],
                           post_delay=0)

        self.add_parameter('hard_homing',
                           label='Hard stop homing start',
                           set_cmd=self.SCL_OPCODE+'HS0',
                           get_cmd=False,
                           get_parser=lambda s: s.partition('=')[2],
                           post_delay=2)

        self.add_parameter('queue_executing',
                           label='Queue Load & Execute',
                           set_cmd=self.SCL_OPCODE+'QX{}',
                           get_cmd=False,
                           vals=vals.Enum(1),  # for time being, only queue 1 is programmed.
                           get_parser=lambda s: s.partition('=')[2],
                           post_delay=2)

        self.add_parameter('encoder_position',
                           label='Read the present encoder position',
                           set_cmd=False,
                           get_cmd=self.SCL_OPCODE+'EP',
                           get_parser=lambda s: (s.partition('=')[2]).partition('{')[0],
                           post_delay=0)


if __name__ == '__main__':
    motor = AmSsm23q(address='192.168.0.130')
    # motor.init_mode(2)
    # print(motor.init_mode())
    # print(motor.alarm_code())
    # print(motor.request_status())
    # print(motor.status_code())
    # print(motor.immediate_analog())
    # print(motor.alarm_reset(None))
    # print(motor.motor_enable(None))
    print(motor.queue_executing(1))
    import numpy as np
    from time import sleep
    for set_point in np.linspace(0, 95000, 2):
        print('current set_point is ' + str(set_point))
        # motor.distance(int(set_point))
        motor.feed_position(set_point)
        sleep(2)
        print('current encoder_position is ' + motor.encoder_position())
        sleep(1)
    # print(motor.distance(20000))
    # print(motor.distance())
    # print(motor.feed_length(None))
    # print(motor.distance(40000))
    # print(motor.distance())
    # print(motor.feed_position(None))



"""
ADAM-6024 module
Refer to the manual page 211-212
"""

from pymodbus.client.sync import ModbusTcpClient
from pymodbus.constants import Endian
from pymodbus.payload import BinaryPayloadBuilder
from time import sleep

builder = BinaryPayloadBuilder(byteorder=Endian.Big)


class Adam6024:
    def __init__(self, host, port):
        self.host = host
        self.port = port

    def __enter__(self):
        self.adam = ModbusTcpClient(self.host, self.port)
        self.adam.connect()
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.adam.close()

    def send_write_ao(self, address, value):
        # scale to 0-10V with 12-bit full range
        payload = round((value/10)*0xFFF)
        return self.adam.write_register(address, payload)

    def send_read_ai(self, address, count=1):
        return self.adam.read_holding_registers(address, count)  # Fixed count 1

    def send_write_do(self, address, value):
        return self.adam.write_coil(address, value)

    def set_ao0(self, value):
        self.send_write_ao(10, value)  # address 10 maps to 4X 40011

    def set_ao1(self, value):
        self.send_write_ao(11, value)  # address 11 maps to 4X 40012

    def get_ai0(self):
        # scale back to -10-10V with 16-bit full range
        payload = self.send_read_ai(0).registers[0]
        value = (payload/0xFFFF)*20-10
        return value

    def set_do0(self, value):
        self.send_write_do(16, value)  # address 16 maps to 0X 00017


if __name__ == '__main__':
    with Adam6024('192.168.1.201', '502') as adam:  # default address at 192.168.1.201
        adam.set_ao0(10)
        sleep(1)
        adam.set_ao1(0)
        sleep(1)
        print(f'The ai0 readout value is::::::: {adam.get_ai0()}')
        sleep(1)
        adam.set_do0(1)
        sleep(1)
        adam.set_do0(0)



""""
BK4065 signal generator from BK PRECISION https://www.bkprecision.com/
First, install USBTMC driver which only requires VISA installed on the computer.
Second, Use pyvisa shell to get usb visa resource name,
python -m visa shell
list
"""
from qcodes import VisaInstrument, validators as vals
from qcodes.instrument.channel import InstrumentChannel, ChannelList


class Bk4065Channel(InstrumentChannel):
    def __init__(self, parent, name, ch_num):
        super().__init__(parent, name)

        def val_parser(input_string):
            """
            Parses return values from instrument. e.g. 'C1: BSWV WVTP,SINE,FRQ,1000,AMP,3,OFST,3,PHSE,0'
            Args:
                input_string: The raw return value
            """

            payload_list = input_string.strip('C1:').strip(' ').strip('BSWV').strip(' ').split(',')
            payload_dict = dict([(k, v) for k, v in zip(payload_list[::2], payload_list[1::2])])
            return payload_dict

        self.add_parameter('basic_wave',
                           label=f'Channel {ch_num} basic wave setting',
                           get_cmd=f'C{ch_num}:BSWV?',
                           get_parser=val_parser)

        self.add_parameter('basic_wave_type',
                           label=f'Channel {ch_num} basic wave type',
                           set_cmd=f'C{ch_num}:BSWV WVTP,{{}}',
                           vals=vals.Enum('SINE', 'SQUARE', 'RAMP', 'PULSE', 'NOISE', 'ARB', 'DC')
                           )

        self.add_parameter('basic_wave_frequency',
                           label=f'Channel {ch_num} basic wave frequency',
                           set_cmd=f'C{ch_num}:BSWV FRQ,{{}}HZ',
                           vals=vals.Numbers(0, 300)
                           )

        self.add_parameter('basic_wave_amplitude',
                           label=f'Channel {ch_num} basic wave amplitude',
                           set_cmd=f'C{ch_num}:BSWV AMP,{{}}V',
                           vals=vals.Numbers(0, 5)
                           )

        self.add_parameter('basic_wave_width',
                           label=f'Channel {ch_num} basic wave pulse width',
                           set_cmd=f'C{ch_num}:BSWV WIDTH,{{}}S',
                           vals=vals.Numbers(0, 5e-6)
                           )

        self.add_parameter('basic_wave_offset',
                           label=f'Channel {ch_num} basic wave offset',
                           set_cmd=f'C{ch_num}:BSWV OFST,{{}}V',
                           vals=vals.Numbers(0, 2.5)
                           )

        self.add_parameter('basic_wave_delay',
                           label=f'Channel {ch_num} basic wave pulse delay',
                           set_cmd=f'C{ch_num}:BSWV DLY,{{}}S',
                           vals=vals.Numbers(0, 5)
                           )

        self.add_parameter('output_enable',
                           label=f'Enable or disable the output channel {ch_num}',
                           set_cmd=f'C{ch_num}:OUTP {{}}',
                           vals=vals.Enum('ON', 'OFF')
                           )

        self.add_parameter('output_load',
                           label=f'Set output load to 50Ohm or Hi-impedance to channel {ch_num}',
                           set_cmd=f'C{ch_num}:OUTP LOAD,{{}}',
                           vals=vals.Enum('50', 'HZ')
                           )


class Bk4065(VisaInstrument):
    """
    This is the QCoDeS driver for the BK PRECISION 4065 signal generator.
    """

    def __init__(self, address, timeout=20, **kwargs):
        """
        Initialises the BK4065.
        Args:
            name (str): Name of the instrument used by QCoDeS
        address (string): Instrument address as used by VISA
            timeout (float): visa timeout, in secs. long default (180)
              to accommodate large waveforms
        """

        # Init VisaInstrument. device_clear MUST NOT be issued, otherwise communications hangs
        # due a bug in firmware
        super().__init__('BK4065', address, device_clear=False, timeout=timeout, **kwargs)
        self.connect_message()

        for channel_number in range(1, 3):
            channel = Bk4065Channel(self, f'ch{channel_number}', channel_number)
            self.add_submodule(f'ch{channel_number}', channel)


if __name__ == '__main__':
    sig_inst2 = Bk4065('USB0::0xF4ED::0xEE3A::448H17107::INSTR',  terminator='\n')
    sig_inst2.ch1.basic_wave_type('PULSE')
    sig_inst2.ch1.basic_wave_frequency(25)  # 25Hz
    sig_inst2.ch1.basic_wave_amplitude(5)  # 5V
    sig_inst2.ch1.basic_wave_offset(2.5)
    sig_inst2.ch1.basic_wave_width(5e-6)  #5uS
    sig_inst2.ch1.output_load('HZ')
    sig_inst2.ch1.output_enable('ON')

    sig_inst2.ch2.basic_wave_type('PULSE')
    sig_inst2.ch2.basic_wave_frequency(25)  # 25Hz
    sig_inst2.ch2.basic_wave_amplitude(5)  # 5V
    sig_inst2.ch2.basic_wave_offset(2.5)
    sig_inst2.ch2.basic_wave_width(5e-6)  #5uS
    sig_inst2.ch2.basic_wave_delay(2e-6)
    sig_inst2.ch2.output_load('HZ')
    sig_inst2.ch2.output_enable('ON')




# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['instru']

package_data = \
{'': ['*']}

install_requires = \
['pymodbus>=2.2,<3.0', 'pyvisa-py>=0.3.1,<0.4.0', 'qcodes>=0.5.2,<0.6.0']

entry_points = \
{'console_scripts': ['demo = instru:main']}

setup_kwargs = {
    'name': 'instru',
    'version': '0.1.6.4',
    'description': 'Various commonly used instrument drivers',
    'long_description': None,
    'author': 'James Maxwell',
    'author_email': None,
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)



from qcodes.instrument_drivers.Keysight.KeysightAgilent_33XXX import WaveformGenerator_33XXX


class Ks33510b(WaveformGenerator_33XXX):
    def __init__(self, ip_address):
        # set pyvisa backend to pyvisa-py, no more NI
        super().__init__('Ks33510b_', f'TCPIP0::{ip_address}::inst0::INSTR', visalib='@py')


if __name__ == '__main__':
    SIG_ADDRESS = '192.168.0.90'
    sig_inst = Ks33510b(SIG_ADDRESS)
    sig_inst.ch1.function_type('PULS')
    sig_inst.ch1.amplitude_unit('VPP')
    sig_inst.ch1.amplitude(5)
    sig_inst.ch1.pulse_width(5e-6)
    sig_inst.ch1.offset(2.5)
    sig_inst.ch1.frequency(25)
    sig_inst.ch1.output('ON')
    sig_inst.sync.source(1)
    sig_inst.error()
